package javax.sql.rowset;

import java.util.PropertyPermission;
import java.security.Permission;
import java.util.Iterator;
import java.util.ServiceConfigurationError;
import java.util.ServiceLoader;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.security.AccessControlException;
import java.sql.SQLException;
import sun.reflect.misc.ReflectUtil;

public class RowSetProvider
{
    private static final String ROWSET_DEBUG_PROPERTY = "javax.sql.rowset.RowSetProvider.debug";
    private static final String ROWSET_FACTORY_IMPL = "com.sun.rowset.RowSetFactoryImpl";
    private static final String ROWSET_FACTORY_NAME = "javax.sql.rowset.RowSetFactory";
    private static boolean debug;
    
    protected RowSetProvider() {
    }
    
    public static RowSetFactory newFactory() throws SQLException {
        RowSetFactory rowSetFactory = null;
        String systemProperty = null;
        try {
            trace("Checking for Rowset System Property...");
            systemProperty = getSystemProperty("javax.sql.rowset.RowSetFactory");
            if (systemProperty != null) {
                trace("Found system property, value=" + systemProperty);
                rowSetFactory = (RowSetFactory)ReflectUtil.newInstance(getFactoryClass(systemProperty, null, true));
            }
        }
        catch (Exception cause) {
            throw new SQLException("RowSetFactory: " + systemProperty + " could not be instantiated: ", cause);
        }
        if (rowSetFactory == null) {
            RowSetFactory loadViaServiceLoader = loadViaServiceLoader();
            rowSetFactory = ((loadViaServiceLoader == null) ? newFactory("com.sun.rowset.RowSetFactoryImpl", null) : loadViaServiceLoader);
            return rowSetFactory;
        }
        return rowSetFactory;
    }
    
    public static RowSetFactory newFactory(String str, ClassLoader obj) throws SQLException {
        trace("***In newInstance()");
        if (str == null) {
            throw new SQLException("Error: factoryClassName cannot be null");
        }
        try {
            ReflectUtil.checkPackageAccess(str);
        }
        catch (AccessControlException cause) {
            throw new SQLException("Access Exception", cause);
        }
        try {
            Class<Object> factoryClass = getFactoryClass(str, obj, false);
            RowSetFactory rowSetFactory = (RowSetFactory)factoryClass.newInstance();
            if (RowSetProvider.debug) {
                trace("Created new instance of " + factoryClass + " using ClassLoader: " + obj);
                return rowSetFactory;
            }
            return rowSetFactory;
        }
        catch (ClassNotFoundException cause2) {
            throw new SQLException("Provider " + str + " not found", cause2);
        }
        catch (Exception ex) {
            throw new SQLException("Provider " + str + " could not be instantiated: " + ex, ex);
        }
    }
    
    private static ClassLoader getContextClassLoader() throws SecurityException {
        return (ClassLoader)AccessController.doPrivileged(new PrivilegedAction<ClassLoader>() {
            @Override
            public ClassLoader run() {
                ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
                if (classLoader == null) {
                    classLoader = ClassLoader.getSystemClassLoader();
                    return classLoader;
                }
                return classLoader;
            }
        });
    }
    
    private static Class<?> getFactoryClass(String name, ClassLoader contextClassLoader, boolean b) throws ClassNotFoundException {
        try {
            if (contextClassLoader == null) {
                contextClassLoader = getContextClassLoader();
                if (contextClassLoader == null) {
                    throw new ClassNotFoundException();
                }
                return contextClassLoader.loadClass(name);
            }
            else {
                return contextClassLoader.loadClass(name);
            }
        }
        catch (ClassNotFoundException ex) {
            if (b) {
                return Class.forName(name, true, RowSetFactory.class.getClassLoader());
            }
            throw ex;
        }
    }
    
    private static RowSetFactory loadViaServiceLoader() throws SQLException {
        RowSetFactory rowSetFactory = null;
        try {
            trace("***in loadViaServiceLoader():");
            Iterator<RowSetFactory> iterator = ServiceLoader.load(RowSetFactory.class).iterator();
            if (iterator.hasNext()) {
                RowSetFactory rowSetFactory2 = (RowSetFactory)iterator.next();
                trace(" Loading done by the java.util.ServiceLoader :" + rowSetFactory2.getClass().getName());
                rowSetFactory = rowSetFactory2;
            }
        }
        catch (ServiceConfigurationError serviceConfigurationError) {
            throw new SQLException("RowSetFactory: Error locating RowSetFactory using Service Loader API: " + serviceConfigurationError, serviceConfigurationError);
        }
        return rowSetFactory;
    }
    
    private static String getSystemProperty(final String s) {
        String s2 = null;
        try {
            s2 = (String)AccessController.doPrivileged(new PrivilegedAction<String>() {
                @Override
                public String run() {
                    return System.getProperty(s);
                }
            }, null, new Permission[] { new PropertyPermission(s, "read") });
        }
        catch (SecurityException obj) {
            trace("error getting " + s + ":  " + obj);
            if (RowSetProvider.debug) {
                obj.printStackTrace();
            }
        }
        return s2;
    }
    
    private static void trace(String str) {
        if (RowSetProvider.debug) {
            System.err.println("###RowSets: " + str);
        }
    }
    
    static {
        RowSetProvider.debug = true;
        String systemProperty = getSystemProperty("javax.sql.rowset.RowSetProvider.debug");
        RowSetProvider.debug = (systemProperty != null && !"false".equals(systemProperty));
    }
}
